#!/usr/local/bin/python

# Copyright (c) 2007 The Regents of the University of California.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#   1. Redistributions of source code must retain the above copyright
#      notice, this list of conditions, and the following disclaimer.
#   2. Redistributions in binary form must reproduce the above
#      copyright notice, this list of conditions, and the following
#      disclaimer in the documentation and/or other materials provided
#      with the distribution.
#   3. Redistributions must acknowledge that this software was
#      originally developed by the UCSF Computer Graphics Laboratory
#      under support by the NIH National Center for Research Resources,
#      grant P41-RR01081.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER "AS IS" AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
# PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
# OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
# EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import os
#import fio
import string
from Input import *
from Image import *
from CD import *
from Tools import *
from Xml import *

###################
# main program

if __name__ == '__main__':
        def usage(prog):
                print "Usage: %s [-o file.xml] [file.in]" % prog
                print "\t-o:\tsend output to file.xml"
                print "\t-n:\tdo not print residues"
                raise SystemExit, 1
        import sys
        import getopt
        frame = 0
        verbose = 0
        no = 0
        output = sys.stdout
        try:
                opts, args = getopt.getopt(sys.argv[1:], "vnfo:")
        except:
                usage(sys.argv[0])
        for opt, val in opts:
                if opt == "-f":
                        frame = 1
                elif opt == "-o":
                        output = val
                elif opt == "-v":
			verbose = 1
                elif opt == "-n":
			no = 1

        if len(args) == 0:
                input = sys.stdin
        elif len(args) == 1:
                input = args[0]
        else:
                usage(sys.argv[0])

	out2 = output

	#determine the input data as well as the
	#direction of the segments
	r = Input(input,verbose)
	prot_name = r.protein_name

	# basic coordinate information
	x = 0.0
	y = 0.0
	R = 5.0
	B = 1
	sl = 3.5
	offset = 16.0*R

	#line ="-1000.0 -2000.0 5000.0 5000.0\n" 
	#outf.write(line)

	# attempt at drawing
	msg = "-|drawing section|-"
 	printVerbose(verbose,msg)

	# determine the beginning section
	r.farend = 0.0
 	j = 0
	k = 0
	if r.numsegs <= 0:
		print "problem with initial data"
	else:
		# initial loop draw
		C1 = r.loops[k].startLoop
		C2 = r.loops[k].endLoop
		chainlen = C2 - C1 + 1
		#msg = "loop %d starts at %d and ends at %d" % (k+1,C1,C2)
	 	#printVerbose(verbose,msg)
		CD.startCD = C1
		CD.endCD = C2
		CD.clenCD = chainlen
		CD.strandCD = r.seq[C1-1:C2]
		CD.numCD = j
		# have data to work with
		if r.side == 1:
			#print "protein starts on the outside"
			if r.loops[j].sideLoop == 1:
				#loop on the top
				#print "this segment needs to be flipped in orientation \n"
				flip = 1
			else:
				#loop on the bottom
				flip = 0

		else:
			#print "protein starts on the inside"
			if r.loops[j].sideLoop == 2:
				#loop on the bottom
				#print "this segment needs to be flipped in orientation \n"
				flip = 0
			else:
				#loop on the top
				flip = 1

		u2 = drawLP(r,CD,x,r.loops[k].sideLoop,flip,verbose)

		k = k + 1
		# work with the chains  
		msg = "working with chains"
		printVerbose(verbose,msg)
		while j < r.numsegs:

			C1 = r.segs[j].startSeg
			C2 = r.segs[j].endSeg
			chainlen = C2 - C1 + 1
			msg = " chain %d starts at %d and ends at %d" % (j+1,C1,C2)
			printVerbose(verbose,msg)
			CD.startCD = C1
			CD.endCD = C2
			CD.clenCD = chainlen
			CD.strandCD = r.seq[C1-1:C2]
			#print "strand is %s" %CD.strandCD
			CD.numCD = j
			if r.segs[j].typeSeg == 1:
				# complete crossing of membrane
				if r.segs[j].dir == 1:
					#segment going from top to bottom
					flip = 0
				else:
					#segment going from bottom to top
					#print "this segment needs to be flipped in orientation \n"
					flip = 1
				u = drawCD(r,CD,x,flip,verbose)
					
			else:
				# partial crossing of membrane
				msg = "into partial area"
				printVerbose(verbose,msg)
				msg = " segment is %d and flip is %d" % (j,flip)
				printVerbose(verbose,msg)

				if r.segs[j].dir == 1:
					#segment going from top to top
					flip = 0
				else:
					#segment going from bottom to bottom
					msg = "this segment needs to be flipped in orientation \n"
					printVerbose(verbose,msg)
					flip = 1

				u = drawCDpart(r,CD,x,flip,verbose)
					
			if r.segs[j].typeSeg == 2:
				x = x + offset			

			if j == r.numsegs -1:
				# stop if number of segments reached
				#print "j is %d and I am quitting" % j
				break

			# work with connecting loops
			CD.startCD = r.loops[k].startLoop 
			C1 = CD.startCD
			CD.endCD = r.loops[k].endLoop 
			C2 = CD.endCD
			CD.clenCD = C2 - C1 + 1
			CD.strandCD = r.seq[C1-1:C2]
			CD.numCD = k
			# fill in information about the connecting loop
			whichside = r.loops[k].sideLoop
			if whichside == 1:
				where = 'outside'
				flip = 1
			else:
				where = 'inside'
				flip = 0

			#print "this loop is on the %s" % where
			#msg = "loop %d starts at %d and ends at %d" % (k+1,C1,C2)
		 	#printVerbose(verbose,msg)
			u2 = drawLP(r,CD,x,r.loops[k].sideLoop,flip,verbose)
			
			j = j + 1
			k = k + 1
			x = x + offset

		# finish up the image
		stop = len(r.seq)
		CD.startCD = r.loops[k].startLoop
		whichside = r.loops[k].sideLoop
		last = r.segs[k-1].typeSeg
		msg = "side is %s and last type is %s" %(whichside,last)
		printVerbose(verbose,msg)
		C1 = CD.startCD
		if C1 <= stop:
			CD.endCD = r.loops[k].endLoop
			C2 = CD.endCD
			CD.clenCD = C2 - C1 + 1
			CD.strandCD = r.seq[C1-1:C2]
			msg = " the last region starts at %d and ends at %d" % (C1,C2)
			printVerbose(verbose,msg)
			
			if r.cusdo == "y":
				#print "k is %d and cusloop last is %d" %(k,r.cusloop[r.cusnum-1]-1)
				if r.cusloop[r.cusnum-1]-1 == k:
					r.cusloop[r.cusnum-1] = 100

			CD.numCD = 99
			if last == 2 and whichside == 2:
				whichside = 1
				r.loops[k].sideLoop = 1
			elif last == 2 and whichside == 1 :
				whichside = 2
				r.loops[k].sideLoop = 2
			if whichside == 1:
				where = 'outside'
				flip = 0
			else:
				where = 'inside'
				flip = 1

			msg = " the end is on the %s" % where
			printVerbose(verbose,msg)
			msg = "r farend is %f" %r.farend
			printVerbose(verbose,msg)

			u2 = drawLP(r,CD,x,r.loops[k].sideLoop,flip,verbose)


	#change the residues marked as being part of a signature
	if r.sigdo == 'y':
		msg = " there are %d signatures" % r.signum
		printVerbose(verbose,msg)

		ncts = 0
		while ncts < r.signum:
			k = 0
			while k < len(r.seq):
				line2 = r.image[k].line
				fields = string.split(line2)
				j = int(fields[0])
				starts = int(r.sigstart[ncts])
				ends = int(r.sigend[ncts])
				if j >= starts and j <= ends:
					if j == starts:
	 					msg = " starts is  %d and end is %d" % (starts,ends)
						printVerbose(verbose,msg)
					l2 = fields[1]
					l3 = string.atof(fields[2])
					l4 = string.atof(fields[3])
					l5 = int(r.sigsym[ncts])
					l6 = r.sigcol[ncts]
        				line = "%d %s %f %f %d %s" %(j,l2,l3,l4,l5,l6)
        				r.image[k].line = line
				k = k + 1
				if k == len(r.seq):
					break
			ncts = ncts + 1


  	#change the residues marked as being special
	if r.spdo == 'y':
		msg = " there are %d special areas" % r.spnum
		printVerbose(verbose,msg)
		ncts = 0
		while ncts < r.spnum:
			k = 0
			while k < len(r.seq):
				line2 = r.image[k].line
				fields = string.split(line2)
				j = int(fields[0])
				pos = int(r.sppos[ncts])
				if j == pos:
					l2 = fields[1]
					l3 = string.atof(fields[2])
					l4 = string.atof(fields[3])
					l5 = int(r.spsym[ncts])
					l6 = r.spcol[ncts]
        				line = "%d %s %f %f %d %s" %(j,l2,l3,l4,l5,l6)
					msg = "special data -|%s|-" %line
					printVerbose(verbose,msg)
        				r.image[k].line = line
				k = k + 1
			ncts = ncts + 1

	#determine the min and max values from image data
	minx = 50000.0
	maxx = -50000.0
	miny = 50000.0
	maxy = -50000.0

	k = 0
	msg = " len is %d"% len(r.seq)
	printVerbose(verbose,msg)

	while k < len(r.seq):		
                line = r.image[k].line

		fields = string.split(line)
		l1 = string.atoi(fields[0])
		if l1 == 0:
			print "k is %d min max line is %s" %(k,line)
		l3 = string.atof(fields[2])
		l4 = string.atof(fields[3])
		if l3 < minx:
			minx = l3
		if l3 > maxx:
			maxx = l3 
		if l4 < miny:
			miny = l4
		if l4 > maxy:
			maxy = l4 
		k = k + 1
		if k == len(r.seq):
			break
	#print "zero is %d" %r.zero
	if r.zero != 0:
		if r.zero < miny:
			miny = r.zero
		if r.zero > maxy:
			maxy = r.zero

	msg = " mins and maxs are %f %f %f %f" % (minx,maxx,miny,maxy)
	printVerbose(verbose,msg)
	if maxx < 50:
		maxx = 50
		msg = 'working with close right side end'
		printVerbose(verbose,msg)
	if minx > -50 and maxx < 50:
	        minx = -50
        	maxx = 50
        	msg = "working with small x range"
		printVerbose(verbose,msg)
	if minx > -50:
	        minx = -50
        	msg = "working with close to zero value xmin"
		printVerbose(verbose,msg)

	rangex = maxx-minx
	borderx = .02*rangex
	rangey = maxy-miny
	bordery = .02*rangey
	msg = " ranges are x %f and y %f" % (rangex,rangey)
	printVerbose(verbose,msg)

	if borderx < 10.0:
		borderx = 10.0
	if bordery < 10.0:
		bordery = 10.0

        Minx = minx - borderx
        Maxx = maxx + borderx
        Miny = miny - bordery
        Maxy = maxy + bordery
	#print "the values are %f %f %f %f" % (Minx,Maxx,Miny,Maxy)

        yTop = 0.0 - .5*(R +B) 
        yBot = yTop - 6.0*((B + sl)*R) +.5*B

	#write the output file	for xml	(inital data)
	TLABEL = "Extracellular"
	BLABEL = "Cytoplasm"
	outf2 = open(out2, 'w+')
	xml_header(outf2,prot_name)
	xml_bbox(outf2,Minx,Maxx,Miny,Maxy)
	xml_layout(outf2,R,TLABEL,BLABEL)
	xml_mem(outf2,minx,maxx,yTop,yBot)
	
	k = 0
        while k < len(r.seq):
                #print r.image[k].line
		fields = string.split(r.image[k].line)
		l1 = string.atoi(fields[0])
		l2 = fields[1]
		l3 = string.atof(fields[2])
		l4 = string.atof(fields[3])
		l5 = string.atoi(fields[4])
		l6 = fields[5]
               	lines = "%d %s %f %f %d %s\n" % (l1,l2,l3,l4,l5,l6)
		if l1 == 0:
			print "lines is %s" %lines
		else:
			xml_data_line(outf2,l1,l2,l3,l4,l5,l6,no)
		k = k + 1

	if len(r.bridge) > 0:
		k2 = 0
		while k2 < len(r.bridge):
			fields = string.split(r.bridge[k2].line)
			b1 = string.atof(fields[0])
			b2 = string.atof(fields[1])
			b3 = string.atof(fields[2])
			b4 = string.atof(fields[3])
			b5 = string.atof(fields[4])
			b6 = string.atof(fields[5])
			b7 = string.atof(fields[6])
			b8 = string.atof(fields[7])
			lines = "%f %f %f %f %f %f %f %f" % (b1,b2,b3,b4,b5,b6,b7,b8)
			xml_bridge(outf2,b1,b2,b3,b4,b5,b6,b7,b8)
			k2 = k2 + 1

	if len(r.feature) > 0:
		k2 = 0
		while k2 < len(r.feature):
			fields = string.split(r.feature[k2].line)
			f1 = string.atof(fields[0])
			f2 = string.atof(fields[1])
			f3 = string.atof(fields[2])
			f4 = string.atof(fields[3])
			f5 = string.atof(fields[4])
			f6 = string.atof(fields[5])
			f7 = string.atof(fields[6])
			f8 = string.atof(fields[7])
			lines = "%f %f %f %f %f %f %f %f" % (f1,f2,f3,f4,f5,f6,f7,f8)
			xml_feature(outf2,f1,f2,f3,f4,f5,f6,f7,f8)
			k2 = k2 + 1

	xml_data_tail(outf2)
	outf2.close()

